/*********************************************************************
 * $Id: alarm.C,v 1.10 2001/05/23 19:28:54 tjko Exp $
 *
 * Copyright (c) 1998,2001  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#include <sys/types.h>
#include <time.h>
#include "main.H"

static char *rcsid = "$Id: alarm.C,v 1.10 2001/05/23 19:28:54 tjko Exp $";

S_Llist *alarms = new S_Llist(1);

Alarm_i::Alarm_i(const Alarm_i *other)
{
  save=other->save;
  h=other->h;
  m=other->m;
  day=other->day;
  month=other->month;
  weekday=other->weekday;
  status=other->status;
  msg=strdup(other->msg);
}

//----------------

void alarm_time_str(char *buf, Alarm_i *a)
{
  char buf2[256];

  if (a->weekday) sprintf(buf2,"%ss",alarm_dayofweek->text(a->weekday));
  else {
    sprintf(buf2,"%2d.%2d.",a->day,a->month);
    
  }
  
  sprintf(buf,"%c%02d:%02d  %10s  %s",(a->save?'*':' '),
	  a->h,a->m,buf2,a->msg);

}

void update_alarm_list()
{
  Alarm_i *a;
  char buf[1024];

  alarm_list->clear();
  if (alarms->size()) {
    a=(Alarm_i*)alarms->begin();
    while (a) {
      alarm_time_str(buf,a);
      alarm_list->add(buf);
      a=(Alarm_i*)a->next();
    }
  }
}


void alarm_check(void *)
{
  Alarm_i *a,*next;
  time_t tsec;
  struct tm *t;
  int i,this_time;

  // fprintf(stderr,"alarm check\n");
  
  if (alarms->size()) {
    tsec=time(NULL);
    t=localtime((const time_t*)&tsec);
    if (!t) { fl_message("localtime() failed!"); exit(-3); };
    this_time=t->tm_hour*60+t->tm_min;

    a=(Alarm_i*)alarms->begin();
    do {
      next=(Alarm_i*)a->next();
      i=a->h*60+a->m;

      if (a->month && a->month!=(t->tm_mon+1)) continue;
      if (a->day && a->day!=(t->tm_mday)) continue;
      if (a->weekday && ( a->weekday!=(t->tm_wday+1) &&
			  !(a->weekday==8 && t->tm_wday>0 && t->tm_wday <6 ) ) 
	  ) continue;

      if ( i<=this_time && ( a->status==0 || 
			     a->status!=(t->tm_mday+100*(t->tm_mon+1)) ) ) {
	fprintf(stderr,"\a"); // fflush(stderr);
	a->status=t->tm_mday+100*(t->tm_mon+1);
	show_alarm_message(a);
	if (!a->save) {
	  alarms->del(a);
	  update_alarm_list();
	}
	// break;
      }
      

    } while ( (a=next) != NULL );

  }

  Fl::add_timeout(60,alarm_check,0);
}



void alarm_cb(Fl_Button *, void *)
{
  if (rcsid);
 
  alarm_hour->value("");
  alarm_minute->value("");
  alarm_day->value("");
  alarm_month->value("");

  alarm_message->value("");

  alarm_dayofweek->clear();
  alarm_dayofweek->add("*");
  alarm_dayofweek->add("Sunday");
  alarm_dayofweek->add("Monday");
  alarm_dayofweek->add("Tuesday");
  alarm_dayofweek->add("Wednesday");
  alarm_dayofweek->add("Thursday");
  alarm_dayofweek->add("Friday");
  alarm_dayofweek->add("Saturday");
  alarm_dayofweek->add("Workday");
  alarm_dayofweek->value(0);

  update_alarm_list();

  alarm_panel->show();
}


void alarm_add_cb(Fl_Button *, void *)
{
  Alarm_i alarm, *a = &alarm;

  if (strlen(alarm_message->value()) < 1) {
    fl_message("Cannot add alarm without message text!");
    return;
  }
  if ( (strlen(alarm_hour->value()) < 1) && 
       (strlen(alarm_minute->value()) < 1)) {
    fl_message("Cannot add alarm without time being set!");
    return;
  }

  sscanf(alarm_hour->value(),"%d",&a->h);
  if (a->h > 23) a->h=23;
  if (a->h < 0) a->h=0;

  sscanf(alarm_minute->value(),"%d",&a->m);
  if (a->m > 59) a->m=59;
  if (a->m < 0) a->m=0;

  sscanf(alarm_day->value(),"%d",&a->day);
  if (a->day > 31) a->day=31;
  if (a->day < 0) a->day=0;

  sscanf(alarm_month->value(),"%d",&a->month);
  if (a->month > 12) a->month=12;
  if (a->month < 0) a->month=0;

  a->weekday=alarm_dayofweek->value();
  a->save=(alarm_permanent->value()?1:0);
  a->msg=strdup(alarm_message->value());

  alarms->push_begin(a);

  update_alarm_list();
}


void alarm_delete_cb(Fl_Button *, void *)
{
  int i = alarm_list->value() -1;

  if (i<0) return;

  alarms->del(i);
  update_alarm_list();
}

int save_alarms(char *fname)
{
  FILE *fp;
  Alarm_i *a;

  if (!fname) return -1;
  fp=fopen(fname,"w");
  if (!fp) return -1;

  fprintf(fp,"# .jyudesk-alarms\n#\n");

  if (alarms->size()) {
    a=(Alarm_i*)alarms->begin();
    while (a) {
      if (a->save)
	fprintf(fp,"%d %d %d %d %d %d : %s\n",a->h,a->m,a->day,a->month,
		a->weekday,a->status,a->msg);
      a=(Alarm_i*)a->next();
    }
  }

  fclose(fp);
  return 0;
}

int load_alarms(char *fname)
{
  FILE *fp;
  char buf[1024],*s;
  Alarm_i alarm, *a = &alarm;

  if (!fname) return -1;
  fp=fopen(fname,"r");
  if (!fp) return -1;

  while (fgets(buf,sizeof(buf),fp)) {
    if (buf[0]=='#') continue;
    s=buf+strlen(buf)-1;
    while (s>=buf && (*s==10 || *s==13)) *(s--)=0;
    if (sscanf(buf,"%d %d %d %d %d %d",&a->h,&a->m,&a->day,&a->month,
	   &a->weekday,&a->status)!=6) continue;
    s=strstr(buf," : ");
    if (!s) continue;
    s+=3;
    a->msg=s;
    a->save=1;
    alarms->add(a);
  }

  a->msg=NULL;
  fclose(fp);
  return 0;
}

void alarm_save_cb(Fl_Button *, void *)
{
  if (save_alarms(alarm_file))
    fl_message("Error saving alarms to file:\n%s",alarm_file);
  else
    fl_message("Permanent alarms saved succesfully to file:\n%s",
	       alarm_file);
}
 

//-------


static void alarm_msg_cb(Fl_Return_Button* o, void*) {
  (o->window())->hide();
  delete (o->window());
}

void show_alarm_message(Alarm_i *i) {
 char buf[256];
 Fl_Window *w;
 {Fl_Window* o = new Fl_Window(589, 148, "jyuDESK alarm!");
  w = o;
  o->labeltype(FL_NORMAL_LABEL);
 }
 w->begin();
 {Fl_Box* o = new Fl_Box(10, 50, 570, 30, strdup(i->msg));
  o->box(FL_EMBOSSED_FRAME);
  o->labeltype(FL_ENGRAVED_LABEL);
  o->labelfont(4);
  o->align(16);
 }
 {Fl_Output* o = new Fl_Output(90, 10, 210, 20, "Alarm time:");
  o->box(FL_FLAT_BOX);
  o->color(FL_GRAY);
  o->color2(FL_GRAY);

  sprintf(buf,"%02d:%02d",i->h,i->m);
  o->value(buf);
 }
 {Fl_Return_Button* o = new Fl_Return_Button(260, 110, 80, 30, "OK");
  o->callback((Fl_Callback*)alarm_msg_cb);
 }
 w->end();

 w->show();
 return;
}
