/*********************************************************************
 * jyudesk -- main program
 * $Id: main.C,v 1.28 2001/05/23 19:30:01 tjko Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <stdarg.h>
#include <signal.h>

#include "main.H"

#define VERSION "v0.7.1"

static char *rcsid = "$Id: main.C,v 1.28 2001/05/23 19:30:01 tjko Exp $";
 
char *xlock_cmd = NULL;
char *keymap1_cmd = NULL;
char *keymap2_cmd = NULL;
char *keymap1_name = NULL;
char *keymap2_name = NULL;
char *keymap_label = NULL;
int  draw_border = 1;
int  RGB_mode = 0;
int  clock1_dif = 0;
int  clock2_dif = 0;
char *clock1_tz = NULL;
char *clock2_tz = NULL;
Fl_Pixmap *bm_mail;
Fl_Pixmap *bm_nomail;

char *conf_file = NULL;
char *mailbox_file = NULL;
char *alarm_file = NULL;

extern char *pop3_pwd;
extern int pop3_enabled;

Fl_Menu menutable[] = {
  {"&Alarm",         FL_ALT+'a', (Fl_Callback*)alarm_cb},
  {"&PhoneBook",     FL_ALT+'p', (Fl_Callback*)show_phone_panel_cb},
  {"&Xlock",         FL_ALT+'x', (Fl_Callback*)xlock_cb},
  {"&Configuration", FL_ALT+'c', config_show_cb},
  {"A&bout...",     FL_ALT+'b', about_cb, 0, FL_MENU_DIVIDER},
  {"&Quit",          FL_ALT+'q', quit_cb},
  {0}
};

//-----------------------------------------------------------------


void clock_tick(void *)
{
  static char buf[255];
  int m = main_clock->minute();
  int h = main_clock->hour();

  int h1 = h + clock1_dif;
  if (h1<0) h1=24-h1;
  if (h1>23) h1=h-24;
  sprintf(buf,"%02d:%02d %3s",h1,m,clock1_tz);
  main_time1->value(buf);

  int h2 = h + clock2_dif;
  if (h2<0) h2=24-h2;
  if (h2>23) h2=h2-24;
  sprintf(buf,"%02d:%02d %3s",h2,m,clock2_tz);
  main_time2->value(buf);

  int n = 60 - main_clock->second() +0;

  Fl::add_timeout(n,clock_tick,0);
}


void raise_flag(int i)
{
  static int mode = 0;

  if (i) {
    // new mail...
    if (mode==0) {
	bm_mail->label(main_mailstatus);
	main_panel->redraw();
	mode=1;
    }
    fprintf(stderr,"\a");
    main_panel->show();
  }
  else {
    // no new mail...
    if (mode==1) {
      bm_nomail->label(main_mailstatus);
      main_panel->redraw();
      mode=0;
    }
  }

}



void mailbox_check(void *)
{
  struct stat buf;
  static time_t mtime = 0;

  if (pop3_enabled) {
    pop3_check();
    return;
  }
  else if (mailbox_file) {
    if (stat(mailbox_file,&buf)==0) {
      if (buf.st_atime < buf.st_mtime) {
	// newmail...
	if (buf.st_mtime > mtime) {
	  raise_flag(1);
	  mtime=buf.st_mtime;
	}
      } else {
	// no new mail...
	raise_flag(0);
      }
    }
  }

  Fl::add_timeout(10,mailbox_check,0);
}

//-----

void xlock_cb(Fl_Button *, void *)
{
  system(xlock_cmd);
}


void keymap1_cb(Fl_Button *, void *)
{
  system(keymap1_cmd);
}

void keymap2_cb(Fl_Button *, void *)
{
  system(keymap2_cmd);
}


void my_atclose(Fl_Window *w, void*)
{
  if (!w || w==main_panel) {
    // fprintf(stderr,"atclose\n");
    if (alarm_file) save_alarms(alarm_file);
    exit(0);
  } else {
    w->hide();
  }
}


void quit_cb(Fl_Object*, void*)
{
  if (alarm_file) save_alarms(alarm_file);
  exit(0);
}

void about_cb(Fl_Object*,void*)
{
  fl_message("jyuDESK %s\n%s\n%s", VERSION, HOST_TYPE,
	     "Copyright (c) 1998-1999  Timo Kokkonen  <tjko@iki.fi>");
}

void show_info(char *format, ...)
{
  va_list args;
  static char buf[8000];  // Hack....

  va_start(args,format);
  vsprintf(buf,format,args);
  va_end(args);

  main_info->value(buf);
}

void own_signal_handler(int sig)
{
  fprintf(stderr,"jyudesk: caught a signal %d\n",sig);
  exit(0);
}

//-----------------------------------------------------------------

int main(int argc, char **argv)
{
  char *s,*fname;
  int w,h;

  if (rcsid);

  create_main_panel();

  if ((s=getenv("HOME"))!=NULL) {
    fname = new char[strlen(s)+12];
    alarm_file = new char[strlen(s)+20];
    strcpy(fname,s);
    strcat(fname,"/.jyudesk");
    strcpy(alarm_file,s);
    strcat(alarm_file,"/.jyudesk-alarms");
  } else {
    fname = strdup(".jyudesk");
    alarm_file = strdup(".jyudesk-alarms");
  }

  if ((s=getenv("MAIL"))!=NULL) mailbox_file = strdup(s);



  conf_file = fname;
  load_config(conf_file);
  load_alarms(alarm_file);

  signal(SIGHUP,own_signal_handler);
  signal(SIGTERM,own_signal_handler);
  signal(SIGINT,own_signal_handler);

  bm_mail=get_bitmap(2);
  bm_nomail=get_bitmap(1);

  S_Socket::ignore_pipe(); // ignore SIGPIPE 

  // add menu
  w=main_panel->w();
  h=main_panel->h();
  Fl_Menu_Button mb(0,0,w,h);
  //mb.box(0);
  mb.type(Fl_Menu_Button::POPUP3);
  mb.menu(menutable);
  main_panel->add(mb);
  bm_nomail->label(main_mailstatus);

  if (!draw_border) main_panel->clear_border();
  if (RGB_mode) Fl::visual(FL_RGB);
  Fl::set_atclose(my_atclose);

  main_panel->show(argc,argv);

  if (pop3_enabled) 
    pop3_pwd=ask_passwd();
  if (!pop3_pwd) pop3_enabled=0;

  Fl::add_timeout(1,clock_tick,0);
  Fl::add_timeout(2,temperature_update,0);
  Fl::add_timeout(5,mailbox_check,0);
  Fl::add_timeout(30,alarm_check,0);

  while (1) Fl::wait();
}
