/*********************************************************************
 * jyudesk -- main program
 * $Id: pop3.C,v 1.6 1998/08/04 15:35:05 tjko Exp $
 *
 * Copyright (c) 1998  Timo Kokkonen <tjko@iki.fi>
 *
 *
 * This file may be copied under the terms and conditions 
 * of the GNU General Public License, as published by the Free
 * Software Foundation (Cambridge, Massachusetts).
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "main.H"
#include "md5.h"

static char *rcsid = "$Id: pop3.C,v 1.6 1998/08/04 15:35:05 tjko Exp $";

#define POP3_PORT "110"
#define TIMEOUT 10

char *pop3_host = NULL;
char *pop3_user = NULL;
char *pop3_pwd = NULL;
int pop3_enabled = 0;

S_Socket pop3_socket, test_socket;
S_Socket *p_sock=&test_socket, *pop_sock=&pop3_socket;
                                

char *get_msg_id(const char *s)
{
  char *p1,*p2,*p3,*n;

  while ((p1=strstr(s,"<"))!=NULL) {
    p2=strstr(p1,"@");
    p3=strstr(p1,">");
    if (!p2 || !p3) continue;
    if (p3 < p2) continue;
    n = new char[p3-p1+1 +1];
    strncpy(n,p1,p3-p1+1);
    n[p3-p1+1]=0;
    return n;
  }
  return NULL;
}

void pop3_passwd_cb(Fl_Button *, void *)
{
  char *t;
  t=ask_passwd();
  if (!t) return;
  if (strlen(t)<1) {
    fl_message("Too short password!");
    return;
  }
  
  free(pop3_pwd);
  pop3_pwd=t;
}


static void add_list(char *s)
{
  if (s) pop_panel_list->add(s);
  Fl::wait();
}



#define POIS(s) { add_list((s)); add_list("Test failed!"); \
                  p_sock->close(); return; }

void pop3_test_cb(Fl_Button *, void *)
{
  char buf[256],*msgid,md5[33];
  unsigned char digest[16];
  MD5Context MD5ctx, *md5ctx=&MD5ctx;
  int msgs=0;

  if (rcsid);

  free(pop3_host); pop3_host=strdup(conf_pop_host->value());
  free(pop3_user); pop3_user=strdup(conf_pop_user->value());
  pop3_enabled=(conf_pop_r2->value()?1:0);

  
  if (!pop3_pwd) {
    fl_message("Set password first!");
    return;
  }
  
  pop_panel_list->clear();
  pop_panel->show();
  
  show_info("Hostname lookup...(%s)",pop3_host);
  Fl::wait();
  if (p_sock->setaddress(pop3_host)<0) {
    show_info("");
    sprintf(buf,"Invalid address: %s !",pop3_host);
    add_list(buf);
    return;
  }
  show_info("");

  if (p_sock->setport(POP3_PORT)<0) {
    sprintf(buf,"Invalid port: %d !!!",POP3_PORT);
    add_list(buf);
    return;
  }

  if (p_sock->open() <= 0) {
    sprintf(buf,"Cannot open socket! %s",p_sock->errstr());
    add_list(buf);
    return;
  }
  add_list("Connected to POP3 server...");
  
  if (p_sock->readyforread(TIMEOUT)<1) 
    POIS("Timeout while reading from socket!");
  p_sock->getstr(buf,sizeof(buf));
  pop_panel_list->add(buf);
  msgid = get_msg_id(buf);
  if (!msgid) POIS("Cannot find msg-id, probably not APOP capable server!");

  // fprintf(stderr,"msg-id: '%s'\n",msgid);
  MD5Init(md5ctx);
  MD5Update(md5ctx,(unsigned char*)msgid,strlen(msgid));
  MD5Update(md5ctx,(unsigned char*)pop3_pwd,strlen(pop3_pwd));
  MD5Final(digest,md5ctx);
  MD5tostr(md5,digest);

  sprintf(buf,"APOP %s %s",pop3_user,md5);
  add_list(buf);
  strcat(buf,"\r\n");
  p_sock->write(buf,strlen(buf));
  if (p_sock->readyforread(TIMEOUT)<1) 
    POIS("Timeout while reading from socket!");
  p_sock->getstr(buf,sizeof(buf));
  add_list(buf);
  if (!(strstr(buf,"+OK")==buf)) POIS("Login failed!");


  sprintf(buf,"STAT\r\n");
  add_list("STAT");
  p_sock->write(buf,strlen(buf));
  if (p_sock->readyforread(TIMEOUT)<1) 
    POIS("Timeout while reading from socket!");
  p_sock->getstr(buf,sizeof(buf));
  add_list(buf);
  if (strstr(buf,"+OK")==buf) {
    int i;
    if (sscanf(buf+3,"%d",&i)==1) msgs=i;
  }
	   
  if (msgs>0) {
    sprintf(buf,"UIDL %d",msgs);
    add_list(buf); strcat(buf,"\r\n");
    p_sock->write(buf,strlen(buf));
    if (p_sock->readyforread(TIMEOUT)<1) 
      POIS("Timeout while reading from socket!");
    p_sock->getstr(buf,sizeof(buf));
    add_list(buf);
    if (strstr(buf,"+OK")==buf) add_list("Server supports UIDL command.");
    else add_list("Server does not support UIDL command.");
  
    sprintf(buf,"TOP %d 0",msgs);
    add_list(buf); strcat(buf,"\r\n");
    p_sock->write(buf,strlen(buf));
    if (p_sock->readyforread(TIMEOUT)<1) 
      POIS("Timeout while reading from socket!");
    p_sock->getstr(buf,sizeof(buf));
    add_list(buf);
    if (strstr(buf,"+OK")==buf) add_list("Server supports TOP command.");
    else add_list("Server does not support TOP command.");
  }


  add_list("Test finished.");
  p_sock->close();

}




int pop3_open(S_Socket *sock, char *host, char *port)
{
  static char *old_port = NULL, *old_ip = NULL;


  if (!old_ip || strcmp(old_ip,host)) {
    show_info("Hostname lookup...(%s)",host);
    Fl::wait();
    if (sock->setaddress(host)<0) {
      fprintf(stderr,"jyudesk: Invalid address: %s\n",host);
      show_info("");
      return 1;
    }
    show_info("");
    Fl::wait();
    if (old_ip) free(old_ip);
    old_ip=strdup(host);
  }
  if (!old_port || strcmp(old_port,port)) {
    if (sock->setport(port)<0) {
      fprintf(stderr,"jyudesk: Invalid port: %s\n",port);
      return 1;
    }
    if (old_port) free(old_port);
    old_port=strdup(port);
  }
  if (sock->open(1)<=0) {
    fprintf(stderr,"jyudesk: Cannot open socket!\n");
    return 1;
  }

  return 0;
}



void pop3_callback(int, void*)
{
  pop3_check();
}


#define POP3_READ(x)  { \
    wasread=pop_sock->read(data+data_res,BUF_SIZE-data_res); \
    if (wasread >= 0) { \
      data_res+=wasread; \
      data[data_res]=0; \
      if ( (strstr(data,x)) || data_res>=BUF_SIZE || \
           strstr(data,"-ERR")==data) { \
	/* fprintf(stderr,"'%s'\n",data); */ \
      } else { \
	if (pop_sock->error()==EAGAIN) { \
	  Fl::add_fd(pop_sock->fd(),FL_READ ,pop3_callback,NULL); \
	  return; \
	} \
      } \
    } else { \
      fprintf(stderr,"jyudesk: Cannot read socket!\n"); \
      goto pois; \
    } \
  }



#define BUF_SIZE 4000

void pop3_check()
{
  static char *old_msgid=NULL;
  static int state = 0;
  static char data[BUF_SIZE+1];
  static int data_res;
  int wasread;
  char *msgid,*status;
  char md5[33];
  unsigned char digest[16];
  MD5Context MD5ctx, *md5ctx=&MD5ctx;

  // fprintf(stderr,"POP3_check() called...%d %d\n",time(NULL),state);

  switch (state) {

  case 0:
    if (pop3_open(pop_sock,pop3_host,POP3_PORT)) goto pois;
    state = 1;
    Fl::add_fd(pop_sock->fd(),FL_READ,pop3_callback,NULL);
    show_info("Checking mailbox...");
    data_res=0;
    return;

    
  case 1: 
    Fl::remove_fd(pop_sock->fd());
    POP3_READ("\n");

    msgid=get_msg_id(data);
    if (!msgid) {
      fprintf(stderr,"jyudesk: not '%s' a POP3/APOP server...\n",pop3_host);
      goto pois;
    }
    
    // fprintf(stderr,"msgid: %s\n",msgid);

    if (!pop3_pwd) {
      fprintf(stderr,"jyudesk: No POP3 password given!\n");
      goto pois;
    }

    MD5Init(md5ctx);
    MD5Update(md5ctx,(unsigned char*)msgid,strlen(msgid));
    MD5Update(md5ctx,(unsigned char*)pop3_pwd,strlen(pop3_pwd));
    MD5Final(digest,md5ctx);
    MD5tostr(md5,digest);
    sprintf(data,"APOP %s %s\r\n",pop3_user,md5);
    // fprintf(stderr,"%s\n",data);
    pop_sock->write(data,strlen(data));
    state=2;
    data_res=0;
    Fl::add_fd(pop_sock->fd(),FL_READ,pop3_callback,NULL);
    return;


  case 2:
    Fl::remove_fd(pop_sock->fd());
    POP3_READ("\n");
    if (strstr(data,"+OK")!=data) {
      fprintf(stderr,"jyudesk: login failed to POP3 server!\n");
      goto pois;
    }
    
    sprintf(data,"STAT\r\n");
    pop_sock->write(data,strlen(data));
    state=3;
    data_res=0;
    Fl::add_fd(pop_sock->fd(),FL_READ,pop3_callback,NULL);
    return;
    
  case 3:
    Fl::remove_fd(pop_sock->fd());
    POP3_READ("\n");
    int i;
    if (strstr(data,"+OK")!=data || sscanf(data+3,"%d",&i)!=1) {
      fprintf(stderr,"jyudesk: POP3 command STAT failed!\n");
      goto pois;
    }
    
    sprintf(data,"TOP %d 0\r\n",i);
    pop_sock->write(data,strlen(data));
    state=4;
    data_res=0;
    Fl::add_fd(pop_sock->fd(),FL_READ,pop3_callback,NULL);
    return;

  case 4:
    Fl::remove_fd(pop_sock->fd());
    POP3_READ(".\r\n");

    status=strstr(data,"Status:");
    msgid=strstr(data,"Message-Id:");
    if (!msgid) msgid=strstr(data,"Message-ID:");
    
    if (!msgid) msgid="N/A";
    else {
      msgid=strstr(msgid,":")+2;
      char *s=msgid;
      while (*s && *s!='\r' && *s!='\n') s++;
      *s=0;
    }
    
    if (!status) status="U";
    else {
      status=strstr(status,":")+2;
      char *s=status;
      while (*s && *s!='\r' && *s!='\n') s++;
      *s=0;
    }
		   
    // fprintf(stderr,"status=%s msgid=%s\n",status,msgid);

    if (strstr(status,"U") && (!old_msgid||strcmp(msgid,old_msgid))) 
      raise_flag(1);
    else 
      raise_flag(0);

    if (old_msgid) free(old_msgid);
    old_msgid=strdup(msgid);

  default:
    goto pois;
  };
  

 pois:
  Fl::add_timeout(60,mailbox_check,0);
  show_info("");
  pop_sock->close();
  state=0;
  return;
}
